// reading/writing functions

gmexport double ds_bin_get_cell(double x, double y, double z) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	return bin_get(selectedbin,xx,yy,zz);
}

gmexport double ds_bin_set_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,(uint32_t)(val));
	return 1;
}

// math functions

gmexport double ds_bin_add_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,bin_get(selectedbin,xx,yy,zz)+(uint32_t)(val));
	return 1;
}

gmexport double ds_bin_subtract_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,bin_get(selectedbin,xx,yy,zz)-(uint32_t)(val));
	return 1;
}

gmexport double ds_bin_invsubtract_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,(uint32_t)(val)-bin_get(selectedbin,xx,yy,zz));
	return 1;
}

gmexport double ds_bin_multiply_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,bin_get(selectedbin,xx,yy,zz)*(uint32_t)(val));
	return 1;
}

gmexport double ds_bin_divide_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,bin_get(selectedbin,xx,yy,zz)/(uint32_t)(val));
	return 1;
}

gmexport double ds_bin_invdivide_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,(uint32_t)(val)/bin_get(selectedbin,xx,yy,zz));
	return 1;
}

// expression functions

gmexport double ds_bin_equal_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)==(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

gmexport double ds_bin_notequal_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)!=(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

gmexport double ds_bin_greaterthan_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)>(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

gmexport double ds_bin_greaterthanorequal_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)>=(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

gmexport double ds_bin_lessthan_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)<(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

gmexport double ds_bin_lessthanorequal_cell(double x, double y, double z, double val, double trueval, double falseval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz,((bin_get(selectedbin,xx,yy,zz)<=(uint32_t)(val))?((uint32_t)(trueval)):((uint32_t)(falseval))));
	return 1;
}

// replace-expression functions

gmexport double ds_bin_replaceequal_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)==(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

gmexport double ds_bin_replacenotequal_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)!=(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

gmexport double ds_bin_replacegreaterthan_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)>(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

gmexport double ds_bin_replacegreaterthanorequal_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)>=(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

gmexport double ds_bin_replacelessthan_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)<(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

gmexport double ds_bin_replacelessthanorequal_cell(double x, double y, double z, double val, double replaceval) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	if(bin_get(selectedbin,xx,yy,zz)<=(uint32_t)(val)) {
		bin_set(selectedbin,xx,yy,zz,(uint32_t)(replaceval));
	}
	return 1;
}

// (unary) boolean functions

gmexport double ds_bin_boolean_not_cell(double x, double y, double z) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, !( bin_get(selectedbin,xx,yy,zz) ) );
	return 1;
}

// (unary) bitwise functions

gmexport double ds_bin_bitwise_not_cell(double x, double y, double z) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, ~( bin_get(selectedbin,xx,yy,zz) ) );
	return 1;
}

gmexport double ds_bin_bitwise_lshift_cell(double x, double y, double z, double bits) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, bin_get(selectedbin,xx,yy,zz) << (unsigned char)(bits) );
	return 1;
}

gmexport double ds_bin_bitwise_rshift_cell(double x, double y, double z, double bits) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, bin_get(selectedbin,xx,yy,zz) >> (unsigned char)(bits) );
	return 1;
}

// boolean functions (not useful here)

// bitwise functions

gmexport double ds_bin_bitwise_and_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, bin_get(selectedbin,xx,yy,zz) & (uint32_t)(val) );
	return 1;
}

gmexport double ds_bin_bitwise_or_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, bin_get(selectedbin,xx,yy,zz) | (uint32_t)(val) );
	return 1;
}

gmexport double ds_bin_bitwise_xor_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, bin_get(selectedbin,xx,yy,zz) ^ (uint32_t)(val) );
	return 1;
}

gmexport double ds_bin_bitwise_nand_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, ~( bin_get(selectedbin,xx,yy,zz) & (uint32_t)(val) ) );
	return 1;
}

gmexport double ds_bin_bitwise_nor_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, ~( bin_get(selectedbin,xx,yy,zz) | (uint32_t)(val) ) );
	return 1;
}

gmexport double ds_bin_bitwise_nxor_cell(double x, double y, double z, double val) {
	unsigned long xx,yy,zz;
	if(selectedbin==NULL) return -1;
	xx = (unsigned long) x;
	yy = (unsigned long) y;
	zz = (unsigned long) z;
	if(!(xx<selectedbin->x && yy<selectedbin->y && zz<selectedbin->z)) {
		return 0;
	}
	bin_set(selectedbin,xx,yy,zz, ~( bin_get(selectedbin,xx,yy,zz) ^ (uint32_t)(val) ) );
	return 1;
}

