// copy functions

gmexport double ds_bin_copy_direct(double dest_id, double src_id) {
	ds_bin *dest,*src;
	if((dest=search_bin((unsigned long) dest_id))==NULL) return -1;
	if((src=search_bin((unsigned long) src_id))==NULL) return -1;
	if(!(dest->bits==src->bits && dest->x==src->x && dest->y==src->y && dest->z==src->z)) {
		return 0;
	}
	if(dest!=src) { // I am not sure if memcpy will work if the pointers are the same
		memcpy(dest->data,src->data,dest->size);
	}
	return 1;
}

gmexport double ds_bin_copy(double dest_id, double src_id) {
	unsigned long xx,yy,zz;
	ds_bin *dest,*src;
	if((dest=search_bin((unsigned long) dest_id))==NULL) return -1;
	if((src=search_bin((unsigned long) src_id))==NULL) return -1;
	if(!(dest->x==src->x && dest->y==src->y && dest->z==src->z)) {
		return 0;
	}
	for(zz=0;zz<dest->z;++zz) {
		for(yy=0;yy<dest->y;++yy) {
			for(xx=0;xx<dest->x;++xx) {
				bin_set(dest,xx,yy,zz,bin_get(src,xx,yy,zz));
			}
		}
	}
	return 1;
}

gmexport double ds_bin_copy_part(double dest_id, double src_id, double dest_x, double dest_y, double dest_z, double src_x, double src_y, double src_z, double size_x, double size_y, double size_z)
{
	ds_bin *dest,*src;
	unsigned long xx,yy,zz, x1,y1,z1, x2,y2,z2, dx,dy,dz, sx,sy,sz;
	bool copy = false;

	if((dest=search_bin((unsigned long) dest_id))==NULL) return -1;
	
	if (dest_id == src_id)
	{
		src_id = ds_bin_create(dest->bits, dest->x, dest->y, dest->z);
		ds_bin_copy_direct(src_id, dest_id);
		copy = true;
	}

	if((src=search_bin((unsigned long) src_id))==NULL) return -1;
	if (size_x <= 0 || size_y <= 0 || size_z <= 0) return -1;

	dx = (unsigned long)(dest_x - src_x);
	dy = (unsigned long)(dest_y - src_y);
	dz = (unsigned long)(dest_z - src_z);
	sx = (unsigned long)(size_x);
	sy = (unsigned long)(size_y);
	sz = (unsigned long)(size_z);

	// Bind lower pos to dest grid
	x1 = (0 < dest_x)?dest_x:0 - dx;
	y1 = (0 < dest_y)?dest_y:0 - dy;
	z1 = (0 < dest_z)?dest_z:0 - dz;
	// Bind lower pos to src grid
	x1 = (0 < x1)?x1:0 + dx;
	y1 = (0 < y1)?y1:0 + dy;
	z1 = (0 < z1)?z1:0 + dz;

	x2 = x1 + sx; y2 = y1 + sy; z2 = z1 + sz;

	// Bind upper pos to dest grid
	x2 = (dest->x > x2)?x2:dest->x - dx;
	y2 = (dest->y > y2)?y2:dest->y - dy;
	z2 = (dest->z > z2)?z2:dest->z - dz;
	// Bind upper pos to src grid
	x2 = (src->x > x2)?x2:src->x + dx;
	y2 = (src->y > y2)?y2:src->y + dy;
	z2 = (src->z > z2)?z2:src->z + dz;

	if (x1 >= x2 || y1 >= y2 || z1 >= z2) return -1;

	for (zz = z1; zz < z2; zz++)
	for (yy = y1; yy < y2; yy++)
	for (xx = x1; xx < x2; xx++)
	{
		bin_set(dest, xx + dx, yy + dy, zz + dz, bin_get(src, xx, yy, zz));
	}

	if (copy)
	{
		ds_bin_destroy(src_id);
	}

	return 0;
}